==> tecfy_corev2_package-1.1.42.tar.gz: lib/views/components/general/multi_dropdown_component.dart <==
part of tecfy_corev2_client_package;

class MultiDropdownComponent extends StatefulWidget {
  final Entity entity;
  final String? url;
  final void Function(Map<String, dynamic>?)? onClick;
  final void Function(List<dynamic>?) onChange;
  final dynamic urlFilterData;
  final Map<String, dynamic> data;
  final PropertyMultiDropdown property;

  const MultiDropdownComponent(
      {Key? key,
      required this.entity,
      this.url,
      this.onClick,
      this.urlFilterData,
      required this.data,
      required this.property,
      required this.onChange})
      : super(key: key);

  @override
  State<MultiDropdownComponent> createState() => _MultiDropdownComponentState();
}

class _MultiDropdownComponentState extends State<MultiDropdownComponent> {
  List<EntityFilter>? _filters;
  List<dynamic>? _data;
  List<dynamic>? _dataFilterd;
  late Entity _entity;
  int _pageIndex = 0;
  bool _noMoreData = false;
  Map<String, dynamic>? parent;

  @override
  void initState() {
    _entity = widget.entity;
    print(widget.property.entityName);
    //print(entity);
    _getList().then((result) {
      if (result.success) {
        if (parent != null && result.data?.length == 0) {
          widget.onClick?.call(parent);
          return;
        }
        _data = result.data
                ?.where((d) =>
                    !widget.property.isFilter ||
                    widget.data[widget.property.filterValueField] ==
                        d[widget.property.filterField])
                .toList() ??
            [];
        setState(() {
          // print('List =+>  $_data');
          // print('Selected List =+>  ${widget.selectedList}');
          // _data!.forEach((e) {
          //   // e['_id'] = e['_id'].toString();
          //   // print(e['_id'] is String);
          //   print(widget.selectedList.contains(e));
          // });

          // widget.selectedList.forEach((e) {
          //   // e['_id'] = e['_id'].toString();
          //   // print(e['_id'] is String);
          //   // print(widget.selectedList.contains(e));
          // });
        });
      } else
        result.showMessage(context);
    });
    super.initState();
  }

  Future<OperationResult<List<Map<String, dynamic>>?>> _getList() async {
    try {
      if (kDebugMode)
        print(
            '+++++++++++ ${widget.property.filterField} , ${widget.property.filterValueField},  ${widget.data[widget.property.filterValueField]}');
      if (widget.url == null || widget.url!.length == 0) {
        if (!widget.property.isFilter) {
          if (kDebugMode) print('no filter');
          if (_entity.isTree)
            return await EntityService.getDataEx(widget.property.entityName,
                activeOnly: true,
                allRecords: true,
                filterParentId: true,
                parentId: parent?['_id']);
          else
            return await EntityService.getDataEx(
              widget.property.entityName,
              activeOnly: true,
            );
        } else {
          if (widget.property.filterField.isNotEmpty) {
            _filters = [
              EntityFilter(
                  operatorId: 'E',
                  value: widget.data[widget.property.filterValueField],
                  property: _entity.properties.firstWhere((element) =>
                      element.dbName == widget.property.filterField))
            ];
            return await EntityService.getData(_entity,
                pageIndex: 0, activeOnly: true, filter: _filters);
          } else if (widget.property.filterQuery.isNotEmpty) {
            _filters = widget.property.filterQuery
                .split('&')
                .map<EntityFilter>((e) {
                  var op = 'E';
                  var opO = '=';
                  if (e.contains('=')) {
                    op = 'E';
                    opO = '=';
                  }
                  if (e.contains('!=')) {
                    op = 'NE';
                    opO = '!=';
                  }
                  if (e.contains('>')) {
                    op = 'G';
                    opO = '>';
                  }
                  if (e.contains('>=')) {
                    op = 'GE';
                    opO = '>=';
                  }
                  if (e.contains('<')) {
                    op = 'L';
                    opO = '<';
                  }
                  if (e.contains('<=')) {
                    op = 'LE';
                    opO = '<=';
                  }
                  var splitValue = e.split(opO);
                  return EntityFilter(
                      operatorId: op,
                      value:
                          EntityService.getValue(widget.data, splitValue.last),
                      property: _entity.properties.firstWhere((element) =>
                          element.dbName ==
                          EntityService.lowerFirst(splitValue.first)));
                })
                .where((e) => e.value != null)
                .toList();
            return await EntityService.getData(_entity,
                filter: _filters, activeOnly: true);
          } else if (widget.property.filterField.isEmpty &&
              widget.property.filterQuery.isEmpty) {
            if (widget.property.filterValueField.toString().contains('.')) {
              var parentPropName =
                  widget.property.filterValueField.split('.')[0];
              var childPropName =
                  widget.property.filterValueField.split('.')[1];

              var parentData =
                  widget.data[widget.property.lowerFirst(parentPropName)];
              if (kDebugMode)
                print('------ Parent Data $parentPropName $parentData');
              var childData =
                  parentData[widget.property.lowerFirst(childPropName)];
              if (kDebugMode)
                print('------ Child Data $childPropName $childData');
              var parentEntity = EntityService.getEntity(parentPropName);
              var childProperty = parentEntity?.properties.firstWhere(
                  (e) => e.dbName.toLowerCase() == childPropName.toLowerCase());
              if (childProperty is PropertyOtherEntityMany) {
                if (childData == null) {
                  _filters = [
                    EntityFilter(
                        operatorId: "E",
                        propertyName: parentEntity!.dbName + "Id",
                        value: parentData["_id"])
                  ];
                  var result = await EntityService.getData(
                      EntityService.getEntity(childProperty.entityName),
                      activeOnly: true,
                      filter: _filters);
                  if (result.success &&
                      result.data != null &&
                      result.data!.length > 0 &&
                      childProperty.entityName.toLowerCase() !=
                          widget.entity.dbName.toLowerCase()) {
                    if (kDebugMode)
                      print('------------ got data ${result.data!.length}');
                    var childEntity =
                        EntityService.getEntity(childProperty.entityName);
                    var childRelationProperty = childEntity!.properties
                        .firstWhere((e) =>
                            e.propertyType == PropertyTypes.Dropdown &&
                            (e as PropertyDropdown).entityName.toLowerCase() ==
                                widget.entity.dbName.toLowerCase());
                    if (kDebugMode) print(result.data);
                    List<Map<String, dynamic>> lst = [];
                    for (var d in result.data ?? []) {
                      var dd = d[childRelationProperty.dbName.substring(
                          0, childRelationProperty.dbName.indexOf("Id"))];
                      if (dd == null) {
                        dd = await EntityService.getOne(widget.entity.dbName,
                            d[childRelationProperty.dbName]);
                      }
                      if (dd != null) lst.add(dd);
                    }
                    result.data = lst;
                  } else {
                    print('XXXXXXXXXXXXXXXXXXXXXXX not valid data');
                    print(result.success);
                    print(childProperty.entityName);
                    print(widget.entity.dbName);
                    print(result.data);
                  }
                  return result;
                } else {
                  if (kDebugMode) print('--------------- child data not null');
                  var childEntity =
                      EntityService.getEntity(childProperty.entityName);
                  var childRelationProperty = childEntity!.properties
                      .firstWhere((e) =>
                          e.propertyType == PropertyTypes.Dropdown &&
                          (e as PropertyDropdown).entityName.toLowerCase() ==
                              widget.entity.dbName.toLowerCase());
                  List<Map<String, dynamic>> lst = [];
                  if (childData is String) {
                    childData = jsonDecode(childData);
                  }
                  for (var d in childData) {
                    var dd = d[childRelationProperty.dbName.substring(
                        0, childRelationProperty.dbName.indexOf("Id"))];
                    if (dd == null) {
                      dd = await EntityService.getOne(widget.entity.dbName,
                          d[childRelationProperty.dbName]);
                    }
                    if (dd != null) lst.add(dd);
                  }
                  childData = lst;
                  return OperationResult(success: true, data: childData);
                }
              } else {
                if (kDebugMode)
                  print('------------------prop not other many to many!');
              }
            } else {
              if (kDebugMode)
                print('---------------filter field doesn noit has (.)');
              // TODO handel this case
              return OperationResult(success: false);
            }
          }
        }
      } else {
        if (kDebugMode)
          print('================ Filter dropdown from URL =========');
        if (widget.urlFilterData == null)
          return OperationResult(success: false);
        var url = widget.url ?? '';
        for (var p in widget.urlFilterData.keys) {
          if (kDebugMode)
            print('p=$p     value= ${widget.urlFilterData[p].toString()}');
          url = url.replaceAll('{$p}', widget.urlFilterData[p].toString());
        }
        return await TecfyBasicApp.apiService.httpGetDynamic(url)
            as OperationResult<List<Map<String, dynamic>>?>;
      }
      return OperationResult(success: false);
    } catch (err, trace) {
      print('XXXXXXXXXXXXX entity search list -> getList XXXXXXXXXX');
      print(err);
      print(trace);
      return OperationResult(success: false, message: err.toString());
    }
  }

  @override
  Widget build(BuildContext context) {
    return _data != null
        ? MultiSelectComponent(
            title: widget.property.name,
            items: _data!,
            onChange: (List<dynamic>? list) {
              widget.data[widget.property.dbName] = list;
              widget.onChange(list?.map<dynamic>((e) => e as dynamic).toList());
            },
            initalSelect:
                MapService.getValue(widget.data, widget.property.dbName),
            labelPropertyName:
                (_data!.first as Map).containsKey('name') ? 'name' : 'title',
          )
        : CircularProgressIndicator.adaptive();
  }
}

==> tecfy_corev2_package-1.1.49.tar.gz: lib/views/components/general/multi_dropdown_component.dart <==
part of tecfy_corev2_client_package;

class MultiDropdownComponent extends StatefulWidget {
  final Entity entity;
  final String? url;
  final void Function(Map<String, dynamic>?)? onClick;
  final void Function(List<dynamic>?) onChange;
  final dynamic urlFilterData;
  final Map<String, dynamic> data;
  final PropertyMultiDropdown property;

  const MultiDropdownComponent(
      {Key? key,
      required this.entity,
      this.url,
      this.onClick,
      this.urlFilterData,
      required this.data,
      required this.property,
      required this.onChange})
      : super(key: key);

  @override
  State<MultiDropdownComponent> createState() => _MultiDropdownComponentState();
}

class _MultiDropdownComponentState extends State<MultiDropdownComponent> {
  List<EntityFilter>? _filters;
  List<dynamic>? _data;
  List<dynamic>? _dataFilterd;
  late Entity _entity;
  int _pageIndex = 0;
  bool _noMoreData = false;
  Map<String, dynamic>? parent;

  @override
  void initState() {
    _entity = widget.entity;
    print(widget.property.entityName);
    //print(entity);
    _getList().then((result) {
      if (result.success) {
        if (parent != null && result.data?.length == 0) {
          widget.onClick?.call(parent);
          return;
        }
        _data = result.data
                ?.where((d) =>
                    !widget.property.isFilter ||
                    widget.data[widget.property.filterValueField] ==
                        d[widget.property.filterField])
                .toList() ??
            [];
        setState(() {
          // print('List =+>  $_data');
          // print('Selected List =+>  ${widget.selectedList}');
          // _data!.forEach((e) {
          //   // e['_id'] = e['_id'].toString();
          //   // print(e['_id'] is String);
          //   print(widget.selectedList.contains(e));
          // });

          // widget.selectedList.forEach((e) {
          //   // e['_id'] = e['_id'].toString();
          //   // print(e['_id'] is String);
          //   // print(widget.selectedList.contains(e));
          // });
        });
      } else
        result.showMessage(context);
    });
    super.initState();
  }

  Future<OperationResult<List<Map<String, dynamic>>?>> _getList() async {
    try {
      if (kDebugMode)
        print(
            '+++++++++++ ${widget.property.filterField} , ${widget.property.filterValueField},  ${widget.data[widget.property.filterValueField]}');
      if (widget.url == null || widget.url!.length == 0) {
        if (!widget.property.isFilter) {
          if (kDebugMode) print('no filter');
          if (_entity.isTree)
            return await EntityService.getDataEx(widget.property.entityName,
                activeOnly: true,
                allRecords: true,
                filterParentId: true,
                parentId: parent?['_id']);
          else
            return await EntityService.getDataEx(widget.property.entityName,
                activeOnly: true, allRecords: true);
        } else {
          if (widget.property.filterField.isNotEmpty) {
            _filters = [
              EntityFilter(
                  operatorId: 'E',
                  value: widget.data[widget.property.filterValueField],
                  property: _entity.properties.firstWhere((element) =>
                      element.dbName == widget.property.filterField))
            ];
            return await EntityService.getData(_entity,
                pageIndex: 0,
                activeOnly: true,
                filter: _filters,
                allRecords: true);
          } else if (widget.property.filterQuery.isNotEmpty) {
            _filters = widget.property.filterQuery
                .split('&')
                .map<EntityFilter>((e) {
                  var op = 'E';
                  var opO = '=';
                  if (e.contains('=')) {
                    op = 'E';
                    opO = '=';
                  }
                  if (e.contains('!=')) {
                    op = 'NE';
                    opO = '!=';
                  }
                  if (e.contains('>')) {
                    op = 'G';
                    opO = '>';
                  }
                  if (e.contains('>=')) {
                    op = 'GE';
                    opO = '>=';
                  }
                  if (e.contains('<')) {
                    op = 'L';
                    opO = '<';
                  }
                  if (e.contains('<=')) {
                    op = 'LE';
                    opO = '<=';
                  }
                  var splitValue = e.split(opO);
                  return EntityFilter(
                      operatorId: op,
                      value:
                          EntityService.getValue(widget.data, splitValue.last),
                      property: _entity.properties.firstWhere((element) =>
                          element.dbName ==
                          EntityService.lowerFirst(splitValue.first)));
                })
                .where((e) => e.value != null)
                .toList();
            return await EntityService.getData(_entity,
                filter: _filters, activeOnly: true, allRecords: true);
          } else if (widget.property.filterField.isEmpty &&
              widget.property.filterQuery.isEmpty) {
            if (widget.property.filterValueField.toString().contains('.')) {
              var parentPropName =
                  widget.property.filterValueField.split('.')[0];
              var childPropName =
                  widget.property.filterValueField.split('.')[1];

              var parentData =
                  widget.data[widget.property.lowerFirst(parentPropName)];
              if (kDebugMode)
                print('------ Parent Data $parentPropName $parentData');
              var childData =
                  parentData[widget.property.lowerFirst(childPropName)];
              if (kDebugMode)
                print('------ Child Data $childPropName $childData');
              var parentEntity = EntityService.getEntity(parentPropName);
              var childProperty = parentEntity?.properties.firstWhere(
                  (e) => e.dbName.toLowerCase() == childPropName.toLowerCase());
              if (childProperty is PropertyOtherEntityMany) {
                if (childData == null) {
                  _filters = [
                    EntityFilter(
                        operatorId: "E",
                        propertyName: parentEntity!.dbName + "Id",
                        value: parentData["_id"])
                  ];
                  var result = await EntityService.getData(
                      EntityService.getEntity(childProperty.entityName),
                      activeOnly: true,
                      allRecords: true,
                      filter: _filters);
                  if (result.success &&
                      result.data != null &&
                      result.data!.length > 0 &&
                      childProperty.entityName.toLowerCase() !=
                          widget.entity.dbName.toLowerCase()) {
                    if (kDebugMode)
                      print('------------ got data ${result.data!.length}');
                    var childEntity =
                        EntityService.getEntity(childProperty.entityName);
                    var childRelationProperty = childEntity!.properties
                        .firstWhere((e) =>
                            e.propertyType == PropertyTypes.Dropdown &&
                            (e as PropertyDropdown).entityName.toLowerCase() ==
                                widget.entity.dbName.toLowerCase());
                    if (kDebugMode) print(result.data);
                    List<Map<String, dynamic>> lst = [];
                    for (var d in result.data ?? []) {
                      var dd = d[childRelationProperty.dbName.substring(
                          0, childRelationProperty.dbName.indexOf("Id"))];
                      if (dd == null) {
                        dd = await EntityService.getOne(widget.entity.dbName,
                            d[childRelationProperty.dbName]);
                      }
                      if (dd != null) lst.add(dd);
                    }
                    result.data = lst;
                  } else {
                    print('XXXXXXXXXXXXXXXXXXXXXXX not valid data');
                    print(result.success);
                    print(childProperty.entityName);
                    print(widget.entity.dbName);
                    print(result.data);
                  }
                  return result;
                } else {
                  if (kDebugMode) print('--------------- child data not null');
                  var childEntity =
                      EntityService.getEntity(childProperty.entityName);
                  var childRelationProperty = childEntity!.properties
                      .firstWhere((e) =>
                          e.propertyType == PropertyTypes.Dropdown &&
                          (e as PropertyDropdown).entityName.toLowerCase() ==
                              widget.entity.dbName.toLowerCase());
                  List<Map<String, dynamic>> lst = [];
                  if (childData is String) {
                    childData = jsonDecode(childData);
                  }
                  for (var d in childData) {
                    var dd = d[childRelationProperty.dbName.substring(
                        0, childRelationProperty.dbName.indexOf("Id"))];
                    if (dd == null) {
                      dd = await EntityService.getOne(widget.entity.dbName,
                          d[childRelationProperty.dbName]);
                    }
                    if (dd != null) lst.add(dd);
                  }
                  childData = lst;
                  return OperationResult(success: true, data: childData);
                }
              } else {
                if (kDebugMode)
                  print('------------------prop not other many to many!');
              }
            } else {
              if (kDebugMode)
                print('---------------filter field doesn noit has (.)');
              // TODO handel this case
              return OperationResult(success: false);
            }
          }
        }
      } else {
        if (kDebugMode)
          print('================ Filter dropdown from URL =========');
        if (widget.urlFilterData == null)
          return OperationResult(success: false);
        var url = widget.url ?? '';
        for (var p in widget.urlFilterData.keys) {
          if (kDebugMode)
            print('p=$p     value= ${widget.urlFilterData[p].toString()}');
          url = url.replaceAll('{$p}', widget.urlFilterData[p].toString());
        }
        return await TecfyBasicApp.apiService.httpGetDynamic(url)
            as OperationResult<List<Map<String, dynamic>>?>;
      }
      return OperationResult(success: false);
    } catch (err, trace) {
      print('XXXXXXXXXXXXX entity search list -> getList XXXXXXXXXX');
      print(err);
      print(trace);
      return OperationResult(success: false, message: err.toString());
    }
  }

  @override
  Widget build(BuildContext context) {
    return _data != null
        ? MultiSelectComponent(
            title: widget.property.name,
            items: _data!,
            onChange: (List<dynamic>? list) {
              widget.data[widget.property.dbName] = list;
              widget.onChange(list?.map<dynamic>((e) => e as dynamic).toList());
            },
            initalSelect:
                MapService.getValue(widget.data, widget.property.dbName),
            labelPropertyName:
                (_data!.first as Map).containsKey('name') ? 'name' : 'title',
          )
        : CircularProgressIndicator.adaptive();
  }
}
